function f = GARCH(y)

% Estimate a GARCH(1,1) model.
% The input is a (T,1) vector of returns.
% The output contains the likelihood value, the parameter estimates, the exitflag, and the 
% time series of estimated variances.

T = size(y,1);
options = optimset('LargeScale', 'off', 'Display', 'off', 'MaxFunEvals', 500, 'MaxIter', 200 , 'TolFun', 1e-10, 'TolX', 1e-10);

% Initialise variance.
LAMBDA    = 0.7;
expsmooth = LAMBDA .^ [0: 1 : T-1] * y .^ 2;
sigma2    = LAMBDA ^ T * (y'*y/T) + (1-LAMBDA) * expsmooth;

np = 3; % Number of parameters in the GARCH model
% Find the best starting values.
nInitialVectors      = [2000*np, np]; % Number of initial vector fed in the uniform random number generator SAV and GARCH models.
nInitialCond         = 4; % Select the number of initial conditions for the optimisation.
ParamEstimates       = zeros(nInitialCond, np);
initialTargetVectors = unifrnd(0, 1, nInitialVectors); 
fvalTemp             = zeros(nInitialVectors(1), 1);
for i = 1:nInitialVectors(1)
    fvalTemp(i,1) = GARCHlik(initialTargetVectors(i,:), y, sigma2);
end
Results          = [fvalTemp, initialTargetVectors];
SortedResults    = sortrows(Results,1);
BestInitialCond  = SortedResults(1:nInitialCond,2:np+1);

% Run the optimisation algorithm for each of the initial conditions.
for i = 1:size(BestInitialCond,1)
    [ParamEstimates(i,:), fval(i), exitflag(i)] = fminsearch('GARCHlik', BestInitialCond(i,:), options, y, sigma2);
    for kk = 1:10
        [ParamEstimates(i,:), fval(i), exitflag(i)] = fminsearch('GARCHlik', ParamEstimates(i,:), options, y, sigma2);          
        if exitflag(i) == 1
            break
        end
    end
end

% Pick the estimate with the best likelihood.
SortedFval  = sortrows([fval', ParamEstimates, exitflag', BestInitialCond], 1);
f.fvalsmplx = SortedFval(1, 1);
f.theta     = SortedFval(1, 2:np+1)';
%f.exitflag  = SortedFval(1, 5);

% Compute the time series of estimated variances.
f.variance = varianceLoop(f.theta, y, sigma2, 3, np); 
if ~isempty(find(f.variance<=0))
    2
end
